#!/usr/bin/perl -w  

use strict;  
use XML::LibXML; 
use Getopt::Long; 
use Archive::Extract;  
use Archive::Zip;

#http://ahinea.com/en/tech/perl-unicode-struggle.html
binmode STDOUT, ":utf8";

my $workspace; 
my $cat;  
my $hideFixed = '';
my $result = GetOptions ("workspace=s"=>\$workspace,                         
							"cat=s"=>\$cat ,                         
							"hide-fixed"=>\$hideFixed );  
				
if (! $result ){         
	print "Usage: [--workspace=path] [--cat=name] [--hide-fixed]\n\n";         
	print "Default workspace is \$HOME/workspace/\n";         
	print "Ex.: parse.pl \t(lists categories)\n";         
	print "Ex.: parse.pl --cat=reactor \t(lists tasks in category 'reactor')\n";
	print "Ex.: parse.pl --workspace=/home/user/workspace/.metadata/.mylyn\n";         
	print "Ex.: parse.pl --workspace=/home/chivertont/workspace-i386/.metadata/.mylyn --cat=reactor\n";        
	exit 1; 
}  
if (!$workspace){
	$workspace=$ENV{'HOME'}."/workspace/"; 
}  

$workspace=$workspace."/.metadata/.mylyn";

#extract the tasklist from zip of XML 
my $archive = Archive::Extract->new( archive => $workspace.'/tasks.xml.zip' ); 
if (!$archive){         
	print "Error: Failed to load Mylyn tasks, is workspace correct ?\n"; 
	print "Looked in $workspace\n";         
	exit 2; 
}  

my $ok = $archive->extract( to => '/tmp' ) or die $archive->error;  

my $xmlfile='/tmp/tasklist.xml'; 
my $parser = XML::LibXML->new(); 
my $tree ; 
my $root ; 

$tree = $parser->parse_file($xmlfile); 
$root = $tree->getDocumentElement;  

if (!$cat){         print "Categories:\n";         
	foreach my $catEntry ($root->findnodes('/TaskList/TaskCategory')) {
		print $catEntry->getAttribute('Name')."\n";         
	} 
}else{         
	print $cat."\n";         
	print "---------------------------------\n";         
	foreach my $catEntry ($root->findnodes('/TaskList/TaskCategory')) {  
		if ($catEntry->getAttribute('Name') eq $cat){
			foreach my $ref ($catEntry->findnodes('TaskReference')){
				my $handle= $ref->getAttribute('Handle');                                     
				printFmtTask( $root->findnodes("/TaskList/Task[\@Handle='$handle']") , $hideFixed, '' ); 
				#TODO cope with sub-sub-tasks etc          
				foreach my $subtask ($root->findnodes("/TaskList/Task[\@Handle='$handle']/SubTask")){              
					my $ref= $subtask->findvalue("\@Handle");
					printFmtTask( $root->findnodes("/TaskList/Task[\@Handle='$ref']") , $hideFixed, "\\--" );
				}                          
			}                 
		}         
	} 
}  

unlink($xmlfile); 

sub printFmtTask{
	my $logEntry=shift;
	my $hideFixed=shift;     
	my $prefix=shift;
	my $lable=$logEntry->findvalue("\@Label");         
	my $uid=$logEntry->findvalue("\@TaskId");         
	my $endDate=            $logEntry->findvalue("\@EndDate");         
	my $creationDate=       $logEntry->findvalue("\@CreationDate");          
 
	if ($hideFixed eq "" || 
		( 
			$hideFixed ne "" && $endDate eq ''
		)
		){	
		if ($prefix){print $prefix;}
		print "$uid (".fmtDate($creationDate)."):";         
		if ($endDate ne ''){                 
			print "(fixed ".fmtDate($endDate).")";        
		}         
		print " $lable\n";
	} 
}  
	
sub fmtDate{         
	my $str=shift;         
	my @s=split(' ',$str);         
	return $s[0]; 
}

=head1 Information

mylyn-report produces simple reports of tasks from an Eclipse Mylyn task list 'local' repository (located via. workspace path).
It either lists the folders (category) or if given a category name produces a basic report  

=head2 Usage

mylyn-report.pl [--workspace=path] [--cat=name] [--hide-fixed]
         
The default workspace is $HOME/workspace/         

=head2 Install

Mylyn-report requires the following standard Perl modules. Either use your package manager
or run C<perl -MCPAN -eshell> and type C<install foo> where foo is each of

=over

=item * XML::LibXML 

=item * Getopt::Long 

=item * Archive::Extract  

=item * Archive::Zip

=back

=head2 Examples

=over

=item C<mylyn-report.pl>

lists all available categories 
        
=item C<mylyn-report.pl --cat=reactor>        
         
lists tasks in category 'reactor' 

=item C<mylyn-report.pl --cat=reactor --hide-fixed>        
         
lists only outstanding tasks in category 'reactor'

=back

As first two above, but with alternative Eclipse workspace path

C<mylyn-report.pl --workspace=/tmp/workspace2/.metadata/.mylyn>
         
C<mylyn-report.pl --workspace=/tmp/workspace2/.metadata/.mylyn --cat=reactor>        

=head1 URL

http://rachaelandtom.info/mylyn-report

=head1 Version

This is version 0.7

=head2 History

=item Version 0.7

fix docs and hide-fixed feature

=item Version 0.6

Updated docs

Added --hide-fixed

Tidy up a few things, remove warnings

UTF output without warning

=item Version 0.5

First public release

=head1 Copyright

    Copyright 2009, 20010 Tom Chiverton

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
=cut
